@@ -60,6 +60,7 @@ class Term(object):
 class Ch(object):
     terms = []
     dterms = ['x']
+    __array_priority__ = 2.0
 
     _cached_parms = {}
     _setup_terms = {}
@@ -388,18 +389,26 @@ class Ch(object):
         
         
     def clear_cache(self, itr=None):
-        self._cache['r'] = None
-        self._cache['drs'].clear()
-
-        if itr is None or itr != self._itr:
-            for parent, parent_dict in self._parents.items():
-                if parent._cache['r'] is not None or len(parent._cache['drs']):
-                    parent.clear_cache(itr=itr)
-                object.__setattr__(parent, '_dirty_vars', parent._dirty_vars.union(parent_dict['varnames']))
-                parent._invalidate_cacheprop_names(parent_dict['varnames'])
-
-        object.__setattr__(self, '_itr', itr)
+        todo = [self]
+        done = set([])
+        nodes_visited = 0
+        while len(todo) > 0:
+            nodes_visited += 1
+            next = todo.pop()
+            if itr is not None and itr==next._itr:
+                continue
+            if id(next) not in done:
+                next._cache['r'] = None
+                next._cache['drs'].clear()
+                next._itr = itr
 
+                for parent, parent_dict in next._parents.items():
+                    object.__setattr__(parent, '_dirty_vars', parent._dirty_vars.union(parent_dict['varnames']))
+                    parent._invalidate_cacheprop_names(parent_dict['varnames']) 
+                    todo.append(parent)
+                done.add(id(next))
+        return nodes_visited
+            
         
     def replace(self, old, new):
         if (hasattr(old, 'dterms') != hasattr(new, 'dterms')):
@@ -481,19 +490,18 @@ class Ch(object):
                 else: 
                     raise Exception("Can't set array that is function of arrays.")
 
-            
+            self = self[key]
             dr = self.dr_wrt(inner)
             dr_rev = dr.T
             #dr_rev = np.linalg.pinv(dr)
             inner_shape = inner.shape
 
-
-            t1 = self._superdot(dr_rev, value.ravel())
+            t1 = self._superdot(dr_rev, np.asarray(value).ravel())
             t2 = self._superdot(dr_rev, self._superdot(dr, inner.x.ravel()))
             if sp.issparse(t1): t1 = np.array(t1.todense())
             if sp.issparse(t2): t2 = np.array(t2.todense())
 
-            inner.x += t1.reshape(inner_shape) - t2.reshape(inner_shape)
+            inner.x = inner.x + t1.reshape(inner_shape) - t2.reshape(inner_shape)
             #inner.x = inner.x + self._superdot(dr_rev, value.ravel()).reshape(inner_shape) - self._superdot(dr_rev, self._superdot(dr, inner.x.ravel())).reshape(inner_shape)
 
 
@@ -503,6 +511,12 @@ class Ch(object):
     def __repr__(self):
         return object.__repr__(self) + '\n' + str(self.r)
 
+    def __float__(self):
+        return self.r.__float__()
+
+    def __int__(self):
+        return self.r.__int__()
+
     def on_changed(self, terms):
         pass
         
@@ -563,6 +577,9 @@ class Ch(object):
                     return np.hstack([np.array(lhs.dot(rhs[:,i].todense())) for i in range(rhs.shape[1])])
                 else:
                     return lhs.matmat(rhs)
+            
+            # TODO: Figure out how/whether to do this.
+            #lhs, rhs = utils.convert_inputs_to_sparse_if_possible(lhs, rhs)
 
             if not sp.issparse(lhs) and sp.issparse(rhs):
                 return rhs.T.dot(lhs.T).T
@@ -699,12 +716,29 @@ class Ch(object):
 
         else:
             result = reduce(lambda x, y: x+y, drs)
+
+        # TODO: figure out how/whether to do this.
+        # if result is not None and not sp.issparse(result):
+        #    nonzero = np.count_nonzero(result)
+        #    if nonzero > 0 and hasattr(result, 'size') and result.size / nonzero >= 10.0:
+        #         #import pdb; pdb.set_trace()
+        #         result = sp.csc_matrix(result)
             
             
         if (result is not None) and (not sp.issparse(result)):
             result = np.atleast_2d(result)
         
-        self._cache['drs'][wrt] = result
+        # When the number of parents is one, it indicates that
+        # caching this is probably not useful because not 
+        # more than one parent will likely ask for this same
+        # thing again in the same iteration of an optimization.
+        #
+        # If we *always* filled in the cache, it would require 
+        # more memory but would occasionally save a little cpu,
+        # on average.
+        if len(self._parents.keys()) != 1:
+            self._cache['drs'][wrt] = result
+
         return result
 
 
@@ -716,7 +750,8 @@ class Ch(object):
     ########################################################
     # Visualization
 
-    def show_tree(self):
+    def show_tree(self, cachelim=np.inf):
+        """Cachelim is in Mb. For any cached jacobians above cachelim, they are also added to the graph. """
         
         import tempfile
         import subprocess
@@ -739,9 +774,55 @@ class Ch(object):
                         result += ['%s [label="%s"];' % (src, my_name)]
                         result += ['%s [label="%s"];' % (dst, child_label)]
                         result += string_for(getattr(self, dterm), dterm)
+
+            if cachelim != np.inf and hasattr(self, '_cache') and 'drs' in self._cache:
+                import cPickle as pickle
+                for dtval, jac in self._cache['drs'].items():
+                    # child_label = getattr(dtval, 'label') if hasattr(dtval, 'label') else dterm
+                    # child_label = '%s (%s)' % (child_label, str(dtval.__class__.__name__))
+                    src = 'aaa%d' % (id(self))
+                    dst = 'aaa%d' % (id(dtval))
+                    try:                    
+                        sz = sys.getsizeof(pickle.dumps(jac, -1))
+                    except: # some are functions
+                        sz = 0
+                    # colorattr = "#%02x%02x%02x" % (szpct*255, 0, (1-szpct)*255)
+                    # print colorattr
+                    if sz > (cachelim * 1024 * 1024):
+                        result += ['%s -> %s [style=dotted,color="<<<%d>>>"];' % (src, dst, sz)]
+                    #
+                    # result += ['%s -> %s [style=dotted];' % (src, dst)]
+                    # result += ['%s [label="%s"];' % (src, my_name)]
+                    # result += ['%s [label="%s"];' % (dst, child_label)]
+                    # result += string_for(getattr(self, dterm), dterm)
+                    
             return result
 
         dot_file_contents = 'digraph G {\n%s\n}' % '\n'.join(list(set(string_for(self, 'root'))))
+        if cachelim != np.inf:
+            import re
+            strs = re.findall(r'<<<(\d+)>>>', dot_file_contents, re.DOTALL)
+            if len(strs) > 0:
+                the_max = np.max(np.array([int(d) for d in strs]))
+                for s in strs:
+                    szpct = float(s)/the_max
+                    sz = float(s)
+                    unit = 'b'
+                    if sz > 1024.: 
+                        sz /= 1024
+                        unit = 'K'
+                    if sz > 1024.: 
+                        sz /= 1024
+                        unit = 'M'
+                    if sz > 1024.: 
+                        sz /= 1024
+                        unit = 'G'
+                    if sz > 1024.: 
+                        sz /= 1024
+                        unit = 'T'
+                        
+                    dot_file_contents = re.sub('<<<%s>>>' % s, '#%02x%02x%02x",label="%d%s' % (szpct*255, 0, (1-szpct)*255, sz, unit), dot_file_contents)
+
         dot_file = tempfile.NamedTemporaryFile()
         dot_file.write(dot_file_contents)
         dot_file.flush()
