@@ -16,6 +16,25 @@ import ch
 
 class TestCh(unittest.TestCase):
     
+    def test_cachehits(self):
+        """Test how many nodes are visited when cache is cleared. 
+        If the number of hits changes, it has to be carefully
+        looked at to make sure that correctness and performance
+        don't get messed up by a change."""
+        
+        a = ch.array(1)
+        b = ch.array(2)
+        c = a
+        for i in range(10):
+            c = a + c + b
+    
+        c.dr_wrt(a)
+        c.dr_wrt(b)
+        self.assertEqual(a.clear_cache() + b.clear_cache(), 59)
+        c.dr_wrt(a)
+        c.dr_wrt(b)
+        self.assertEqual(a.clear_cache(123) + b.clear_cache(123), 41)
+    
     def test_nested_concatenate(self):
         aa = ch.arange(3)
         bb = ch.arange(4)
@@ -30,6 +49,11 @@ class TestCh(unittest.TestCase):
         self.assertTrue(result.dr_wrt(bb).nnz > 0)
         self.assertTrue(result.dr_wrt(cc).nnz > 0)
 
+    def test_casting(self):
+        for fn in float, int:
+            self.assertEqual(fn(np.array(5)),     fn(ch.array(5)))
+            self.assertEqual(fn(np.array([[5]])), fn(ch.array([[5]])))
+
     def test_tensordot(self):
         an = np.arange(60.).reshape(3,4,5)
         bn = np.arange(24.).reshape(4,3,2)
@@ -415,6 +439,18 @@ class TestCh(unittest.TestCase):
         self.assertTrue(np.max(np.abs(little.r - np.array([15, 16]))) == 0.)
         _ =  little.dr_wrt(big)
 
+        # Tests assignment through reorderings
+        aa = ch.arange(4*4*4).reshape((4,4,4))[:3,:3,:3]
+        aa[0,1,2] = 100
+        self.assertTrue(aa[0,1,2].r[0] == 100)
+
+        # Tests assignment through reorderings (NaN's are a special case)
+        aa = ch.arange(9).reshape((3,3))
+        aa[1,1] = np.nan
+        self.assertTrue(np.isnan(aa.r[1,1]))
+        self.assertFalse(np.isnan(aa.r[0,0]))
+
+
     def test_redundancy_removal(self):
 
         for MT in [False, True]:
